local Tablet = AceLibrary("Tablet-2.0")
local L = AceLibrary("AceLocale-2.2"):new("FuBar_WhisperFu")
local Dewdrop = AceLibrary("Dewdrop-2.0")

WhisperFu = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0")

WhisperFu:RegisterDB("WhisperFuDB")
WhisperFu:RegisterDefaults('profile', 
	{
		PlayerLimit		= 10,
		MessageLimit	= 10,
		ShowTimes		= false,
		SendColour		= { 0.5, 0.5, 1.0 },
		ReceiveColour	= { 0.0, 1.0, 0.0 },
		data			= { },
		ShowHint		= true,
	}
)

WhisperFu.version = "1.0." .. string.sub("$Revision: 9785 $", 12, -3)
WhisperFu.date = string.sub("$Date: 2006-09-02 10:55:39 +0930 (Sat, 02 Sep 2006) $", 8, 17)
WhisperFu.hasIcon = false
WhisperFu.defaultPosition = 'LEFT'
WhisperFu.hideWithoutStandby = true
WhisperFu.independentProfile = true

-- Data is stored as an array in self.db.profile.data of
-- .name
-- messageList array of
--  { .sent, .time, .message }

function WhisperFu:OnInitialize()
end

function WhisperFu:OnEnable()
	self:RegisterEvent("CHAT_MSG_WHISPER", "OnReceiveWhisper")
	self:RegisterEvent("CHAT_MSG_WHISPER_INFORM", "OnSendWhisper")
	self:UpdateMenu()
end

function WhisperFu:OnDisable()
end

function WhisperFu:GetHexColor(name)
	return string.format("|cff%02x%02x%02x", self.db.profile[name][1] * 255, self.db.profile[name][2] * 255, self.db.profile[name][3] * 255)
end

function WhisperFu:FormatMessage(message, wasSender)
	if wasSender then
		return self:GetHexColor("SendColour")..message;
	else
		return self:GetHexColor("ReceiveColour")..message;
	end
end

function WhisperFu:CreateMessageEntry(wasSender)
	return { sent = wasSender, time = date("%H:%M:%S"), message = arg1 }
end

function WhisperFu:ProcessWhisper(wasSender)
	-- Check that the whisper is from a new person.
	local nameAlreadyInList = false;
	
	for i = 1, self.db.profile.PlayerLimit do
		if(not self.db.profile.data[i]) then
			break
		end
		if(arg2 == self.db.profile.data[i].name) then
			for j = self.db.profile.MessageLimit,2,-1 do
				self.db.profile.data[i].messageList[j] = self.db.profile.data[i].messageList[j-1]
			end
			self.db.profile.data[i].messageList[1] = self:CreateMessageEntry(wasSender)

			-- Move to top of list
			local data = self.db.profile.data[i];
			for j = i,2,-1 do
				self.db.profile.data[j] = self.db.profile.data[j-1];
			end
			self.db.profile.data[1] = data;
			nameAlreadyInList = true;
			break
		end	
	end
	if (nameAlreadyInList == false) then
		-- Move the list down and add new whisperer to the top;
		for i = 10,2,-1 do
			self.db.profile.data[i] = self.db.profile.data[i-1];
		end
		self.db.profile.data[1] = { 
									name = arg2, 
									messageList = { self:CreateMessageEntry(wasSender) }
								  }
	end

	self:UpdateMenu()
end

function WhisperFu:OnReceiveWhisper()
	self:ProcessWhisper(false)
end

function WhisperFu:OnSendWhisper()
	self:ProcessWhisper(true)
end

function WhisperFu:SetSendColour(r, g, b)
	self.db.profile.SendColour[1] = r
	self.db.profile.SendColour[2] = g
	self.db.profile.SendColour[3] = b
	self:UpdateMenu()
end

function WhisperFu:GetSendColour()
	return self.db.profile.SendColour[1], self.db.profile.SendColour[2], self.db.profile.SendColour[3]
end

function WhisperFu:SetReceiveColour(r, g, b)
	self.db.profile.ReceiveColour[1] = r
	self.db.profile.ReceiveColour[2] = g
	self.db.profile.ReceiveColour[3] = b
	self:UpdateMenu()
end

function WhisperFu:GetReceiveColour()
	return self.db.profile.ReceiveColour[1], self.db.profile.ReceiveColour[2], self.db.profile.ReceiveColour[3]
end


function WhisperFu:OnTextUpdate()
	if self.db.profile.data[1] then
		self:SetText( self:FormatMessage( self.db.profile.data[1].name, self.db.profile.data[1].messageList[1].sent ) )
	else
		self:SetText("WhisperFu")
	end
end

function WhisperFu:OnTooltipUpdate()
	if self.db.profile.data[1] then
		local cat
		if self.db.profile.ShowTimes then
			cat = Tablet:AddCategory('text', self.db.profile.data[1].name,
									 'columns', 2)

		else
			cat = Tablet:AddCategory('text', self.db.profile.data[1].name,
									 'columns', 1)
		end
		
		for i=self.db.profile.MessageLimit,1,-1 do
			local messageEntry = self.db.profile.data[1].messageList[i] 
		
			if messageEntry then 			
				local colour
				
				if messageEntry.sent then 
					colour = self.db.profile.SendColour 
				else
					colour = self.db.profile.ReceiveColour
				end
				
				cat:AddLine('text', messageEntry.message,
							'textwrap', true,
							'textR', colour[1],
							'textG', colour[2],
							'textB', colour[3],
							'text2', self:FormatMessage(messageEntry.time, messageEntry.sent))
			end
		end
	end

	if self.db.profile.ShowHint then
		Tablet:SetHint(L["HintText"])
	end
end

function WhisperFu:StartWhisper(player)
   DEFAULT_CHAT_FRAME.editBox.chatType = "WHISPER";
   DEFAULT_CHAT_FRAME.editBox.tellTarget = player;
   DEFAULT_CHAT_FRAME.editBox:Show();
end

function WhisperFu:OnClick()
	if self.db.profile.data[1] then
	
		local playerName = self.db.profile.data[1].name;
		if IsAltKeyDown() then
			InviteByName(playerName);
		elseif IsShiftKeyDown() then
			SendWho("n-"..playerName);
		elseif IsControlKeyDown() then
			TargetByName(playerName);
		else
			self:StartWhisper(playerName)
		end
	end
end

function WhisperFu:PlayerMenu(playerIndex)
	local playerData = self.db.profile.data[playerIndex]

	local playerMenu = 
	{ 
		type = 'group',
		name = self:FormatMessage(playerData.name, playerData.messageList[1].sent),
		desc = string.format(L["PlayerDesc"], playerData.name),
		order = playerIndex,
		args =	
		{
			whisper =	
			{
				type = 'execute',
				name = L["Whisper"],
				desc = string.format(L["WhisperDesc"], playerData.name),
				order = 1,
				func = function() WhisperFu:StartWhisper(playerData.name) end
			},
			
			spacer =
			{
				type = 'header',
				order = 2,
			}
		}
	}
	
	for i=1,self.db.profile.MessageLimit do
		if not playerData.messageList[i] then break end
		
		playerMenu.args["message"..i] = { 
											type = 'header',
											name = self:FormatMessage(playerData.messageList[i].message, playerData.messageList[i].sent),
											order = 10000-i,
										}
	end
	
	return playerMenu
end

function WhisperFu:UpdateMenu()
	local optionsTable = 
	{
		handler = WhisperFu,
		type = 'group',
		args = 
		{
			spacer1 = 
			{
				type = 'header',
				order = 101,
			},

			playerlimit =
			{
				type = 'range',
				name = L["Player Limit"],
				desc = L["Maximum number of players to store messages for."],
				max = 20,
				min = 5,
				step = 1,
				order = 102,
				get = function() return self.db.profile.PlayerLimit end,
				set = function(value) self.db.profile.PlayerLimit = value end
			},
			
			messagelimit =
			{
				type = 'range',
				name = L["Message Limit"],
				desc = L["Maximum number of messages to store per player."],
				max = 20,
				min = 5,
				step = 1,
				order = 103,
				get = function() return self.db.profile.MessageLimit end,
				set = function(value) self.db.profile.MessageLimit = value end
			},
					
			colours = 
			{
				type = 'group',
				name = L["Colours"],
				desc = L["Set the Text Colours"],
				order = 104,
				args = 
				{
					SendColour = 
					{
						type = "color",
						name = L["Send Colour"],
						desc = L["Sets the color of text for whispers sent"],
						get = "GetSendColour",
						set = "SetSendColour",
					},
					ReceiveColour = 
					{
						type = "color",
						name = L["Receive Colour"],
						desc = L["Sets the color of text for whispers receieved"],
						get = "GetReceiveColour",
						set = "SetReceiveColour",
					},
				},
			},
			
			showtimes = 
			{
				type = "toggle",
				order = 105,
				name = L["Show Times"],
				desc = L["Show time of whispers in tooltip."],
				get = function() return self.db.profile.ShowTimes end,
				set = function() self.db.profile.ShowTimes = not self.db.profile.ShowTimes self:UpdateTooltip() end
			},
			
			showHint =
			{
				type = "toggle",
				order = 106,
				name = L["Show Hint"],
				desc = L["Show hint at the bottom of the tooltip."],
				get = function() return self.db.profile.ShowHint end,
				set = function() self.db.profile.ShowHint = not self.db.profile.ShowHint self:UpdateTooltip() end
			},
			
			spacer2 = 
			{
				type = 'header',
				order = 107,
			},

			whisper =	
			{
				type = 'execute',
				name = L["Purge"],
				desc = L["PurgeDesc"],
				order = 108,
				func = function() self:PurgeWhispers() end
			},
			
		},
	}
	
	for i=1,self.db.profile.PlayerLimit do
		if not self.db.profile.data[i] then break end
	
		optionsTable.args["player"..i] = self:PlayerMenu(i)
	end
	
	WhisperFu.OnMenuRequest = optionsTable	
		
	-- Force a menu update.. is there a better way?
	Dewdrop:Unregister(self:GetFrame())
	
	-- UpdateMenu also indicates a need to update tooltip/display
	self:UpdateDisplay()
	
end

function WhisperFu:PurgeWhispers()
	self.db.profile.data = { }
	self:UpdateMenu()
	
	-- This code could be part of the usual UpdateMenu(), but this introduces 'instability' when trying to
	-- view the menus during incoming whispers
	if Dewdrop:IsOpen(self:GetFrame()) then
		Dewdrop:Close()
		self:OpenMenu()
	end
end

function WhisperFu:OnProfileEnable()
	self:UpdateMenu()
	
	-- This code could be part of the usual UpdateMenu(), but this introduces 'instability' when trying to
	-- view the menus during incoming whispers
	if Dewdrop:IsOpen(self:GetFrame()) then
		Dewdrop:Close()
		self:OpenMenu()
	end
end
